@@ -18,12 +18,13 @@ import pandas as pd
 import numpy as np
 from sklearn.base import BaseEstimator
 from sklearn.base import TransformerMixin
-from sklearn.ensemble import GradientBoostingRegressor, GradientBoostingClassifier, RandomForestRegressor, RandomForestClassifier
-from sklearn.linear_model import LassoCV,LogisticRegressionCV
+from sklearn.ensemble import GradientBoostingRegressor, GradientBoostingClassifier, RandomForestRegressor, \
+    RandomForestClassifier
+from sklearn.linear_model import LassoCV, LogisticRegressionCV
 from functools import reduce
 
 
-class RuleCondition():
+class RuleCondition:
     """Class for binary rule condition
 
     Warning: this class should not be used directly.
@@ -34,14 +35,13 @@ class RuleCondition():
                  threshold,
                  operator,
                  support,
-                 feature_name = None):
+                 feature_name=None):
         self.feature_index = feature_index
         self.threshold = threshold
         self.operator = operator
         self.support = support
         self.feature_name = feature_name
 
-
     def __repr__(self):
         return self.__str__()
 
@@ -64,9 +64,9 @@ class RuleCondition():
         X_transformed: array-like matrix, shape=(n_samples, 1)
         """
         if self.operator == "<=":
-            res =  1 * (X[:,self.feature_index] <= self.threshold)
+            res = 1 * (X[:, self.feature_index] <= self.threshold)
         elif self.operator == ">":
-            res = 1 * (X[:,self.feature_index] > self.threshold)
+            res = 1 * (X[:, self.feature_index] > self.threshold)
         return res
 
     def __eq__(self, other):
@@ -76,72 +76,77 @@ class RuleCondition():
         return hash((self.feature_index, self.threshold, self.operator, self.feature_name))
 
 
-class Winsorizer():
+class Winsorizer:
     """Performs Winsorization 1->1*
 
     Warning: this class should not be used directly.
     """
-    def __init__(self,trim_quantile=0.0):
-        self.trim_quantile=trim_quantile
-        self.winsor_lims=None
 
-    def train(self,X):
+    def __init__(self, trim_quantile=0.0):
+        self.trim_quantile = trim_quantile
+        self.winsor_lims = None
+
+    def train(self, X):
         # get winsor limits
-        self.winsor_lims=np.ones([2,X.shape[1]])*np.inf
-        self.winsor_lims[0,:]=-np.inf
-        if self.trim_quantile>0:
+        self.winsor_lims = np.ones([2, X.shape[1]]) * np.inf
+        self.winsor_lims[0, :] = -np.inf
+        if self.trim_quantile > 0:
             for i_col in np.arange(X.shape[1]):
-                lower=np.percentile(X[:,i_col],self.trim_quantile*100)
-                upper=np.percentile(X[:,i_col],100-self.trim_quantile*100)
-                self.winsor_lims[:,i_col]=[lower,upper]
+                lower = np.percentile(X[:, i_col], self.trim_quantile * 100)
+                upper = np.percentile(X[:, i_col], 100 - self.trim_quantile * 100)
+                self.winsor_lims[:, i_col] = [lower, upper]
 
-    def trim(self,X):
-        X_=X.copy()
-        X_=np.where(X>self.winsor_lims[1,:],np.tile(self.winsor_lims[1,:],[X.shape[0],1]),np.where(X<self.winsor_lims[0,:],np.tile(self.winsor_lims[0,:],[X.shape[0],1]),X))
+    def trim(self, X):
+        X_ = np.where(X > self.winsor_lims[1, :], np.tile(self.winsor_lims[1, :], [X.shape[0], 1]),
+                      np.where(X < self.winsor_lims[0, :], np.tile(self.winsor_lims[0, :], [X.shape[0], 1]), X))
         return X_
 
-class FriedScale():
+
+class FriedScale:
     """Performs scaling of linear variables according to Friedman et al. 2005 Sec 5
 
     Each variable is first Winsorized l->l*, then standardised as 0.4 x l* / std(l*)
     Warning: this class should not be used directly.
     """
-    def __init__(self, winsorizer = None):
-        self.scale_multipliers=None
+
+    def __init__(self, winsorizer=None):
+        self.scale_multipliers = None
         self.winsorizer = winsorizer
 
-    def train(self,X):
+    def train(self, X):
         # get multipliers
         if self.winsorizer != None:
-            X_trimmed= self.winsorizer.trim(X)
+            X_trimmed = self.winsorizer.trim(X)
         else:
             X_trimmed = X
 
-        scale_multipliers=np.ones(X.shape[1])
+        scale_multipliers = np.ones(X.shape[1])
         for i_col in np.arange(X.shape[1]):
-            num_uniq_vals=len(np.unique(X[:,i_col]))
-            if num_uniq_vals>2: # don't scale binary variables which are effectively already rules
-                scale_multipliers[i_col]=0.4/(1.0e-12 + np.std(X_trimmed[:,i_col]))
-        self.scale_multipliers=scale_multipliers
+            num_uniq_vals = len(np.unique(X[:, i_col]))
+            if num_uniq_vals > 2:  # don't scale binary variables which are effectively already rules
+                scale_multipliers[i_col] = 0.4 / (1.0e-12 + np.std(X_trimmed[:, i_col]))
+        self.scale_multipliers = scale_multipliers
 
-    def scale(self,X):
+    def scale(self, X):
         if self.winsorizer != None:
-            return self.winsorizer.trim(X)*self.scale_multipliers
+            return self.winsorizer.trim(X) * self.scale_multipliers
         else:
-            return X*self.scale_multipliers
+            return X * self.scale_multipliers
 
 
-class Rule():
+class Rule:
     """Class for binary Rules from list of conditions
 
     Warning: this class should not be used directly.
     """
+
     def __init__(self,
-                 rule_conditions,prediction_value):
+                 rule_conditions, prediction_value):
         self.conditions = set(rule_conditions)
         self.support = min([x.support for x in rule_conditions])
-        self.prediction_value=prediction_value
-        self.rule_direction=None
+        self.prediction_value = prediction_value
+        self.rule_direction = None
+
     def transform(self, X):
         """Transform dataset.
 
@@ -154,10 +159,10 @@ class Rule():
         X_transformed: array-like matrix, shape=(n_samples, 1)
         """
         rule_applies = [condition.transform(X) for condition in self.conditions]
-        return reduce(lambda x,y: x * y, rule_applies)
+        return reduce(lambda x, y: x * y, rule_applies)
 
     def __str__(self):
-        return  " & ".join([x.__str__() for x in self.conditions])
+        return " & ".join([x.__str__() for x in self.conditions])
 
     def __repr__(self):
         return self.__str__()
@@ -187,7 +192,7 @@ def extract_rules_from_tree(tree, feature_names=None):
             rule_condition = RuleCondition(feature_index=feature,
                                            threshold=threshold,
                                            operator=operator,
-                                           support = tree.n_node_samples[node_id] / float(tree.n_node_samples[0]),
+                                           support=tree.n_node_samples[node_id] / float(tree.n_node_samples[0]),
                                            feature_name=feature_name)
             new_conditions = conditions + [rule_condition]
         else:
@@ -202,12 +207,12 @@ def extract_rules_from_tree(tree, feature_names=None):
 
             right_node_id = tree.children_right[node_id]
             traverse_nodes(right_node_id, ">", threshold, feature, new_conditions)
-        else: # a leaf node
-            if len(new_conditions)>0:
-                new_rule = Rule(new_conditions,tree.value[node_id][0][0])
+        else:  # a leaf node
+            if len(new_conditions) > 0:
+                new_rule = Rule(new_conditions, tree.value[node_id][0][0])
                 rules.update([new_rule])
             else:
-                pass #tree only has a root node!
+                pass  # tree only has a root node!
             return None
 
     traverse_nodes()
@@ -215,8 +220,7 @@ def extract_rules_from_tree(tree, feature_names=None):
     return rules
 
 
-
-class RuleEnsemble():
+class RuleEnsemble:
     """Ensemble of binary decision rules
 
     This class implements an ensemble of decision rules that extracts rules from
@@ -235,6 +239,7 @@ class RuleEnsemble():
     rules: List of Rule
         The ensemble of rules extracted from the trees
     """
+
     def __init__(self,
                  tree_list,
                  feature_names=None):
@@ -243,14 +248,14 @@ class RuleEnsemble():
         self.rules = set()
         ## TODO: Move this out of __init__
         self._extract_rules()
-        self.rules=list(self.rules)
+        self.rules = list(self.rules)
 
     def _extract_rules(self):
         """Recursively extract rules from each tree in the ensemble
 
         """
         for tree in self.tree_list:
-            rules = extract_rules_from_tree(tree[0].tree_,feature_names=self.feature_names)
+            rules = extract_rules_from_tree(tree[0].tree_, feature_names=self.feature_names)
             self.rules.update(rules)
 
     def filter_rules(self, func):
@@ -259,7 +264,7 @@ class RuleEnsemble():
     def filter_short_rules(self, k):
         self.filter_rules(lambda x: len(x.conditions) > k)
 
-    def transform(self, X,coefs=None):
+    def transform(self, X, coefs=None):
         """Transform dataset.
 
         Parameters
@@ -273,20 +278,20 @@ class RuleEnsemble():
         X_transformed: array-like matrix, shape=(n_samples, n_out)
             Transformed dataset. Each column represents one rule.
         """
-        rule_list=list(self.rules)
-        if   coefs is None :
+        rule_list = list(self.rules)
+        if coefs is None:
             return np.array([rule.transform(X) for rule in rule_list]).T
-        else: # else use the coefs to filter the rules we bother to interpret
-            res= np.array([rule_list[i_rule].transform(X) for i_rule in np.arange(len(rule_list)) if coefs[i_rule]!=0]).T
-            res_=np.zeros([X.shape[0],len(rule_list)])
-            res_[:,coefs!=0]=res
+        else:  # else use the coefs to filter the rules we bother to interpret
+            res = np.array(
+                [rule_list[i_rule].transform(X) for i_rule in np.arange(len(rule_list)) if coefs[i_rule] != 0]).T
+            res_ = np.zeros([X.shape[0], len(rule_list)])
+            res_[:, coefs != 0] = res
             return res_
+
     def __str__(self):
         return (map(lambda x: x.__str__(), self.rules)).__str__()
 
 
-
-
 class RuleFit(BaseEstimator, TransformerMixin):
     """Rulefit class
 
@@ -331,6 +336,7 @@ class RuleFit(BaseEstimator, TransformerMixin):
         The names of the features (columns)
 
     """
+
     def __init__(
             self,
             tree_size=4,
@@ -350,84 +356,102 @@ class RuleFit(BaseEstimator, TransformerMixin):
             n_jobs=None,
             random_state=None):
         self.tree_generator = tree_generator
-        self.rfmode=rfmode
-        self.lin_trim_quantile=lin_trim_quantile
-        self.lin_standardise=lin_standardise
-        self.winsorizer=Winsorizer(trim_quantile=lin_trim_quantile)
-        self.friedscale=FriedScale(self.winsorizer)
+        self.rfmode = rfmode
+        self.lin_trim_quantile = lin_trim_quantile
+        self.lin_standardise = lin_standardise
+        self.winsorizer = Winsorizer(trim_quantile=lin_trim_quantile)
+        self.friedscale = FriedScale(self.winsorizer)
         self.stddev = None
         self.mean = None
-        self.exp_rand_tree_size=exp_rand_tree_size
-        self.max_rules=max_rules
-        self.sample_fract=sample_fract
-        self.max_rules=max_rules
-        self.memory_par=memory_par
-        self.tree_size=tree_size
-        self.random_state=random_state
-        self.model_type=model_type
-        self.cv=cv
-        self.tol=tol
+        self.exp_rand_tree_size = exp_rand_tree_size
+        self.max_rules = max_rules
+        self.sample_fract = sample_fract
+        self.max_rules = max_rules
+        self.memory_par = memory_par
+        self.tree_size = tree_size
+        self.random_state = random_state
+        self.model_type = model_type
+        self.cv = cv
+        self.tol = tol
         # LassoCV default max_iter is 1000 while LogisticRegressionCV 100.
-        self.max_iter=1000 if 'regress' else 100
-        self.n_jobs=n_jobs
-        self.Cs=Cs
+        if max_iter is not None:
+            self.max_iter = max_iter
+        else:
+            if rfmode == 'regress':
+                self.max_iter = 100
+            else:
+                self.max_iter = 1000
+
+        self.n_jobs = n_jobs
+        self.Cs = Cs
 
     def fit(self, X, y=None, feature_names=None):
         """Fit and estimate linear combination of rule ensemble
 
         """
         ## Enumerate features if feature names not provided
-        N=X.shape[0]
+        N = X.shape[0]
         if feature_names is None:
             self.feature_names = ['feature_' + str(x) for x in range(0, X.shape[1])]
         else:
-            self.feature_names=feature_names
+            self.feature_names = feature_names
         if 'r' in self.model_type:
             ## initialise tree generator
             if self.tree_generator is None:
-                n_estimators_default=int(np.ceil(self.max_rules/self.tree_size))
-                self.sample_fract_=min(0.5,(100+6*np.sqrt(N))/N)
-                if   self.rfmode=='regress':
-                    self.tree_generator = GradientBoostingRegressor(n_estimators=n_estimators_default, max_leaf_nodes=self.tree_size, learning_rate=self.memory_par,subsample=self.sample_fract_,random_state=self.random_state,max_depth=100)
+                n_estimators_default = int(np.ceil(self.max_rules / self.tree_size))
+                self.sample_fract_ = min(0.5, (100 + 6 * np.sqrt(N)) / N)
+                if self.rfmode == 'regress':
+                    self.tree_generator = GradientBoostingRegressor(n_estimators=n_estimators_default,
+                                                                    max_leaf_nodes=self.tree_size,
+                                                                    learning_rate=self.memory_par,
+                                                                    subsample=self.sample_fract_,
+                                                                    random_state=self.random_state, max_depth=100)
                 else:
-                    self.tree_generator =GradientBoostingClassifier(n_estimators=n_estimators_default, max_leaf_nodes=self.tree_size, learning_rate=self.memory_par,subsample=self.sample_fract_,random_state=self.random_state,max_depth=100)
+                    self.tree_generator = GradientBoostingClassifier(n_estimators=n_estimators_default,
+                                                                     max_leaf_nodes=self.tree_size,
+                                                                     learning_rate=self.memory_par,
+                                                                     subsample=self.sample_fract_,
+                                                                     random_state=self.random_state, max_depth=100)
 
-            if   self.rfmode=='regress':
-                if type(self.tree_generator) not in [GradientBoostingRegressor,RandomForestRegressor]:
+            if self.rfmode == 'regress':
+                if type(self.tree_generator) not in [GradientBoostingRegressor, RandomForestRegressor]:
                     raise ValueError("RuleFit only works with RandomForest and BoostingRegressor")
             else:
-                if type(self.tree_generator) not in [GradientBoostingClassifier,RandomForestClassifier]:
+                if type(self.tree_generator) not in [GradientBoostingClassifier, RandomForestClassifier]:
                     raise ValueError("RuleFit only works with RandomForest and BoostingClassifier")
 
             ## fit tree generator
-            if not self.exp_rand_tree_size: # simply fit with constant tree size
+            if not self.exp_rand_tree_size:  # simply fit with constant tree size
                 self.tree_generator.fit(X, y)
-            else: # randomise tree size as per Friedman 2005 Sec 3.3
+            else:  # randomise tree size as per Friedman 2005 Sec 3.3
                 np.random.seed(self.random_state)
-                tree_sizes=np.random.exponential(scale=self.tree_size-2,size=int(np.ceil(self.max_rules*2/self.tree_size)))
-                tree_sizes=np.asarray([2+np.floor(tree_sizes[i_]) for i_ in np.arange(len(tree_sizes))],dtype=int)
-                i=int(len(tree_sizes)/4)
-                while np.sum(tree_sizes[0:i])<self.max_rules:
-                    i=i+1
-                tree_sizes=tree_sizes[0:i]
+                tree_sizes = np.random.exponential(scale=self.tree_size - 2,
+                                                   size=int(np.ceil(self.max_rules * 2 / self.tree_size)))
+                tree_sizes = np.asarray([2 + np.floor(tree_sizes[i_]) for i_ in np.arange(len(tree_sizes))], dtype=int)
+                i = int(len(tree_sizes) / 4)
+                while np.sum(tree_sizes[0:i]) < self.max_rules:
+                    i = i + 1
+                tree_sizes = tree_sizes[0:i]
                 self.tree_generator.set_params(warm_start=True)
-                curr_est_=0
+                curr_est_ = 0
                 for i_size in np.arange(len(tree_sizes)):
-                    size=tree_sizes[i_size]
-                    self.tree_generator.set_params(n_estimators=curr_est_+1)
+                    size = tree_sizes[i_size]
+                    self.tree_generator.set_params(n_estimators=curr_est_ + 1)
                     self.tree_generator.set_params(max_leaf_nodes=size)
                     random_state_add = self.random_state if self.random_state else 0
-                    self.tree_generator.set_params(random_state=i_size+random_state_add) # warm_state=True seems to reset random_state, such that the trees are highly correlated, unless we manually change the random_sate here.
+                    self.tree_generator.set_params(
+                        random_state=i_size + random_state_add)  # warm_state=True seems to reset random_state, such that the trees are highly correlated, unless we manually change the random_sate here.
                     self.tree_generator.get_params()['n_estimators']
                     self.tree_generator.fit(np.copy(X, order='C'), np.copy(y, order='C'))
-                    curr_est_=curr_est_+1
+                    curr_est_ = curr_est_ + 1
                 self.tree_generator.set_params(warm_start=False)
             tree_list = self.tree_generator.estimators_
-            if isinstance(self.tree_generator, RandomForestRegressor) or isinstance(self.tree_generator, RandomForestClassifier):
-                 tree_list = [[x] for x in self.tree_generator.estimators_]
+            if isinstance(self.tree_generator, RandomForestRegressor) or isinstance(self.tree_generator,
+                                                                                    RandomForestClassifier):
+                tree_list = [[x] for x in self.tree_generator.estimators_]
 
             ## extract rules
-            self.rule_ensemble = RuleEnsemble(tree_list = tree_list,
+            self.rule_ensemble = RuleEnsemble(tree_list=tree_list,
                                               feature_names=self.feature_names)
 
             ## concatenate original features and rules
@@ -439,53 +463,51 @@ class RuleFit(BaseEstimator, TransformerMixin):
             ## standard deviation and mean of winsorized features
             self.winsorizer.train(X)
             winsorized_X = self.winsorizer.trim(X)
-            self.stddev = np.std(winsorized_X, axis = 0)
-            self.mean = np.mean(winsorized_X, axis = 0)
+            self.stddev = np.std(winsorized_X, axis=0)
+            self.mean = np.mean(winsorized_X, axis=0)
 
             if self.lin_standardise:
                 self.friedscale.train(X)
-                X_regn=self.friedscale.scale(X)
+                X_regn = self.friedscale.scale(X)
             else:
-                X_regn=X.copy()
+                X_regn = X.copy()
 
         ## Compile Training data
-        X_concat=np.zeros([X.shape[0],0])
+        X_concat = np.zeros([X.shape[0], 0])
         if 'l' in self.model_type:
-            X_concat = np.concatenate((X_concat,X_regn), axis=1)
+            X_concat = np.concatenate((X_concat, X_regn), axis=1)
         if 'r' in self.model_type:
-            if X_rules.shape[0] >0:
+            if X_rules.shape[0] > 0:
                 X_concat = np.concatenate((X_concat, X_rules), axis=1)
 
         ## fit Lasso
-        if self.rfmode=='regress':
-            if self.Cs is None: # use defaultshasattr(self.Cs, "__len__"):
-                n_alphas= 100
-                alphas=None
+        if self.rfmode == 'regress':
+            if self.Cs is None:  # use defaultshasattr(self.Cs, "__len__"):
+                n_alphas = 100
+                alphas = None
             elif hasattr(self.Cs, "__len__"):
-                n_alphas= None
-                alphas=1./self.Cs
+                n_alphas = None
+                alphas = 1. / self.Cs
             else:
-                n_alphas= self.Cs
-                alphas=None
+                n_alphas = self.Cs
+                alphas = None
             self.lscv = LassoCV(
                 n_alphas=n_alphas, alphas=alphas, cv=self.cv,
                 max_iter=self.max_iter, tol=self.tol,
                 n_jobs=self.n_jobs,
                 random_state=self.random_state)
             self.lscv.fit(X_concat, y)
-            self.coef_=self.lscv.coef_
-            self.intercept_=self.lscv.intercept_
+            self.coef_ = self.lscv.coef_
+            self.intercept_ = self.lscv.intercept_
         else:
-            Cs=10 if self.Cs is None else self.Cs
-            self.lscv=LogisticRegressionCV(
-                Cs=Cs, cv=self.cv, penalty='l1', max_iter=self.max_iter,
+            Cs = 10 if self.Cs is None else self.Cs
+            self.lscv = LogisticRegressionCV(
+                Cs=Cs, cv=self.cv, penalty='l2', max_iter=self.max_iter,
                 tol=self.tol, n_jobs=self.n_jobs,
-                random_state=self.random_state, solver='liblinear')
+                random_state=self.random_state, solver='lbfgs')
             self.lscv.fit(X_concat, y)
-            self.coef_=self.lscv.coef_[0]
-            self.intercept_=self.lscv.intercept_[0]
-
-
+            self.coef_ = self.lscv.coef_[0]
+            self.intercept_ = self.lscv.intercept_[0]
 
         return self
 
@@ -493,17 +515,17 @@ class RuleFit(BaseEstimator, TransformerMixin):
         """Predict outcome for X
 
         """
-        X_concat=np.zeros([X.shape[0],0])
+        X_concat = np.zeros([X.shape[0], 0])
         if 'l' in self.model_type:
             if self.lin_standardise:
-                X_concat = np.concatenate((X_concat,self.friedscale.scale(X)), axis=1)
+                X_concat = np.concatenate((X_concat, self.friedscale.scale(X)), axis=1)
             else:
-                X_concat = np.concatenate((X_concat,X), axis=1)
+                X_concat = np.concatenate((X_concat, X), axis=1)
         if 'r' in self.model_type:
-            rule_coefs=self.coef_[-len(self.rule_ensemble.rules):]
-            if len(rule_coefs)>0:
-                X_rules = self.rule_ensemble.transform(X,coefs=rule_coefs)
-                if X_rules.shape[0] >0:
+            rule_coefs = self.coef_[-len(self.rule_ensemble.rules):]
+            if len(rule_coefs) > 0:
+                X_rules = self.rule_ensemble.transform(X, coefs=rule_coefs)
+                if X_rules.shape[0] > 0:
                     X_concat = np.concatenate((X_concat, X_rules), axis=1)
         return self.lscv.predict(X_concat)
 
@@ -513,24 +535,23 @@ class RuleFit(BaseEstimator, TransformerMixin):
         """
 
         if 'predict_proba' not in dir(self.lscv):
-
             error_message = '''
             Probability prediction using predict_proba not available for
             model type {lscv}
             '''.format(lscv=self.lscv)
             raise ValueError(error_message)
 
-        X_concat=np.zeros([X.shape[0],0])
+        X_concat = np.zeros([X.shape[0], 0])
         if 'l' in self.model_type:
             if self.lin_standardise:
-                X_concat = np.concatenate((X_concat,self.friedscale.scale(X)), axis=1)
+                X_concat = np.concatenate((X_concat, self.friedscale.scale(X)), axis=1)
             else:
-                X_concat = np.concatenate((X_concat,X), axis=1)
+                X_concat = np.concatenate((X_concat, X), axis=1)
         if 'r' in self.model_type:
-            rule_coefs=self.coef_[-len(self.rule_ensemble.rules):]
-            if len(rule_coefs)>0:
-                X_rules = self.rule_ensemble.transform(X,coefs=rule_coefs)
-                if X_rules.shape[0] >0:
+            rule_coefs = self.coef_[-len(self.rule_ensemble.rules):]
+            if len(rule_coefs) > 0:
+                X_rules = self.rule_ensemble.transform(X, coefs=rule_coefs)
+                if X_rules.shape[0] > 0:
                     X_concat = np.concatenate((X_concat, X_rules), axis=1)
         return self.lscv.predict_proba(X_concat)
 
@@ -568,39 +589,40 @@ class RuleFit(BaseEstimator, TransformerMixin):
                data set (X)
         """
 
-        n_features= len(self.coef_) - len(self.rule_ensemble.rules)
+        n_features = len(self.coef_) - len(self.rule_ensemble.rules)
         rule_ensemble = list(self.rule_ensemble.rules)
         output_rules = []
         ## Add coefficients for linear effects
         for i in range(0, n_features):
             if self.lin_standardise:
-                coef=self.coef_[i]*self.friedscale.scale_multipliers[i]
+                coef = self.coef_[i] * self.friedscale.scale_multipliers[i]
             else:
-                coef=self.coef_[i]
+                coef = self.coef_[i]
             if subregion is None:
-                importance = abs(coef)*self.stddev[i]
+                importance = abs(coef) * self.stddev[i]
             else:
                 subregion = np.array(subregion)
-                importance = sum(abs(coef)* abs([ x[i] for x in self.winsorizer.trim(subregion) ] - self.mean[i]))/len(subregion)
-            output_rules += [(self.feature_names[i], 'linear',coef, 1, importance)]
+                importance = sum(abs(coef) * abs([x[i] for x in self.winsorizer.trim(subregion)] - self.mean[i])) / len(
+                    subregion)
+            output_rules += [(self.feature_names[i], 'linear', coef, 1, importance)]
 
         ## Add rules
         for i in range(0, len(self.rule_ensemble.rules)):
             rule = rule_ensemble[i]
-            coef=self.coef_[i + n_features]
+            coef = self.coef_[i + n_features]
 
             if subregion is None:
-                importance = abs(coef)*(rule.support * (1-rule.support))**(1/2)
+                importance = abs(coef) * (rule.support * (1 - rule.support)) ** (1 / 2)
             else:
                 rkx = rule.transform(subregion)
-                importance = sum(abs(coef) * abs(rkx - rule.support))/len(subregion)
+                importance = sum(abs(coef) * abs(rkx - rule.support)) / len(subregion)
 
-            output_rules += [(rule.__str__(), 'rule', coef,  rule.support, importance)]
-        rules = pd.DataFrame(output_rules, columns=["rule", "type","coef", "support", "importance"])
+            output_rules += [(rule.__str__(), 'rule', coef, rule.support, importance)]
+        rules = pd.DataFrame(output_rules, columns=["rule", "type", "coef", "support", "importance"])
         if exclude_zero_coef:
             rules = rules.ix[rules.coef != 0]
         return rules
-    
+
     def get_feature_importance(self, exclude_zero_coef=False, subregion=None, scaled=False):
         """
         Returns feature importance for input features to RuleFit model.
@@ -619,8 +641,8 @@ class RuleFit(BaseEstimator, TransformerMixin):
         --------
             return_df (pandas DataFrame): DataFrame for feature names and feature importances (FP 2004 eq. 35)
         """
-        
-        def find_mk(rule:str):
+
+        def find_mk(rule: str):
             """
             Finds the number of features in a given rule from the get_rules method.
 
@@ -632,17 +654,17 @@ class RuleFit(BaseEstimator, TransformerMixin):
             --------
                 var_count (int): 
             """
-            
+
             ## Count the number of features found in a rule
             feature_count = 0
             for feature in self.feature_names:
                 if feature in rule:
                     feature_count += 1
-            return(feature_count)
-        
+            return (feature_count)
+
         feature_set = self.feature_names
         rules = self.get_rules(exclude_zero_coef, subregion)
-        
+
         # Return an array of counts for features found in rules
         features_in_rule = rules.rule.apply(lambda x: find_mk(x))
 
@@ -651,16 +673,16 @@ class RuleFit(BaseEstimator, TransformerMixin):
             # Rules where feature is found
             feature_rk = rules.rule.apply(lambda x: feature in x)
             # Linear importance array for feature
-            linear_imp = rules[(rules.type=='linear')&(rules.rule==feature)].importance.values
+            linear_imp = rules[(rules.type == 'linear') & (rules.rule == feature)].importance.values
             # Rule importance array
-            rule_imp = rules[rules.type!='linear'].importance[feature_rk]
+            rule_imp = rules[rules.type != 'linear'].importance[feature_rk]
             # Total count of features in each rule feature is found
             mk_array = features_in_rule[feature_rk]
-            feature_imp.append(float(linear_imp + (rule_imp/mk_array).sum())) # (FP 2004 eq. 35)
+            feature_imp.append(float(linear_imp + (rule_imp / mk_array).sum()))  # (FP 2004 eq. 35)
 
         # Scaled output
         if scaled:
-            feature_imp = 100*(feature_imp/np.array(feature_imp).max())
-           
-        return_df = pd.DataFrame({'feature':self.feature_names, 'importance':feature_imp})
-        return(return_df)
\ No newline at end of file
+            feature_imp = 100 * (feature_imp / np.array(feature_imp).max())
+
+        return_df = pd.DataFrame({'feature': self.feature_names, 'importance': feature_imp})
+        return (return_df)
